package org.loom.addons.recaptcha;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletRequestImpl;
import org.loom.tags.JspWriterWrapper;
import org.loom.tags.WriterWrapper;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Render a <a href="http://recaptcha.net/">reCAPTCHA</a> challenge field
 * @author icoloma
 */
@Tag(
		bodyContent=BodyContent.EMPTY,
		example="&lt;recaptcha publicKey=\"my-public-key\"/>"
	)
public class RecaptchaTag extends SimpleTagSupport {

	/** request-aware attribute with the ReCaptcha error message code */
	public static final String RECAPTCHA_ERROR_MESSAGE = "recaptcha-message";
	
	/** the recaptcha challenge parameter name */
	public static final String CHALLENGE_PARAM = "recaptcha_challenge_field";
	public static final String RESPONSE_PARAM = "recaptcha_response_field";
	
	public static final String HTTP_SERVER = "api.recaptcha.net";
	public static final String HTTPS_SERVER = "api-secure.recaptcha.net";
	
	/** true to generate noscript tag, default true */
	@Attribute
	private boolean noscript = true;
	
	/** the ReCaptcha public key to use */
	@Attribute(required=true)
	private String publicKey;
	
	@Override
	public void doTag() throws JspException, IOException {
		LoomServletRequest request = LoomServletRequestImpl.getThreadLocal();
		UrlBuilder url = request.createUrl();
		if (request.isSecure()) {
			url.setScheme(UrlBuilder.SECURE_SCHEME);
			url.setHostname(HTTPS_SERVER);
		} else {
			url.setScheme(UrlBuilder.INSECURE_SCHEME);
			url.setHostname(HTTP_SERVER);
		}
		url.setRawUrl("/challenge");
		url.addParameter("k", publicKey);
		url.addParameter("error", request.getAttribute(RECAPTCHA_ERROR_MESSAGE));
	
		WriterWrapper<JspWriter> out = new JspWriterWrapper(getJspContext().getOut());
		out.start("script", "src", url.getURL()).end("script");
	
		if (noscript) {
			out.start("noscript");
			url.setRawUrl("/noscript");
			out.start("iframe", "src", url.getURL(), "height", 300, "width", 500, "frameborder", 0);
			out.end("iframe");
			out.start("textarea", "name", CHALLENGE_PARAM, "rows", 3, "cols", 40).end("textarea");
			out.startEnd("input", "name", RESPONSE_PARAM, "type", "hidden", "value", "manual_challenge");
			out.end("noscript");
		}
 	}

	public void setNoscript(boolean noscript) {
		this.noscript = noscript;
	}

	public void setPublicKey(String publicKey) {
		this.publicKey = publicKey;
	}
	
}
